; KBDSCAN.ASM
; Scan Spectrum keyboard to return raw keycode
; ============================================
; This is similar to the ZXROM code at &028E, except it only tests
; for a single keypress.
; On exit:  CC=no key pressed
;           CS=key pressed
;           A =&00-&27 internal key number
;           BC=corrupted
;
KBDSCAN:    AND  A
            LD   BC,&7FFE    ; C=keyboard port, B=row A8 selected
KBDLINE:    IN   A,(C)       ; Read this row
            CPL              ; Swap 0s to 1s
            AND  &1F         ; Keep key columns
            JR   NZ,KBDPRESS ; <>0, a key is pressed
            RRC  B           ; Select next row
            JR   C,KBDLINE   ; Loop until '0' falls out
            RET              ; CC=no key pressed
KBDPRESS:   LD   C,A         ; Save key row in C
            LD   A,&DF       ; Start at 8-maxkey
KBDLOOP1:   INC  A           ; Inc. key number for each row
            RLC  B           ; Move column towards carry
            JR   C,KBDLOOP1  ; Loop until zero bit falls out
                             ; A=&E0+0..&E0+7 for row A15..A8
KBDLOOP2:   ADD  A,8         ; Increase by eight for each of eight rows
            RLC  C           ; Move row towards carry
            JR   NC,KBDLOOP2 ; Loop until one bit falls out
            RET
;
;                      Keyboard internal keycodes
;      +-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
; A11--| &24 | &1C | &14 | &0C | &04 | &03 | &0B | &13 | &1B | &23 |      --A12
;      |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  |  0  |
;      +-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+
; A10--  | &25 | &1D | &15 | &0D | &05 | &02 | &0A | &12 | &1A | &22 |    --A13
;        |  Q  |  W  |  E  |  R  |  T  |  Y  |  U  |  I  |  O  |  P  |
;        +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
; A9--      | &26 | &1E | &16 | &0E | &06 | &01 | &09 | &11 | &19 | &21 | --A14
;           |  A  |  S  |  D  |  F  |  G  |  H  |  J  |  K  |  L  |ENTER|
;      +----+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+---+-+-----+
; A8-- |  &27   | &1F | &17 | &0F | &07 | &00 | &08 | &10 | &18 |  &20  | --A15
;      | SHIFT  |  Z  |  X  |  C  |  V  |  B  |  N  |  M  | SYM | SPACE |
;      +--------+-----+-----+-----+-----+-----+-----+-----+-----+-------+
;          |       |     |     |     |     |     |     |     |      |
;          D0      D1    D2    D3    D4    D4    D3    D2    D1     D0
;
; The Jupiter keyboard can be scanned with this code, which has a slightly
; different layout.
;                   Jupiter Ace Keyboard internal key codes
;        +-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
; A11--  | &24 | &1C | &14 | &0C | &04 | &03 | &0B | &13 | &1B | &23 |   --A12
;        |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  |  0  |
;        +-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
; A10--  | &25 | &1D | &15 | &0D | &05 | &02 | &0A | &12 | &1A | &22 |   --A13
;        |  Q  |  W  |  E  |  R  |  T  |  Y  |  U  |  I  |  O  |  P  |
;        +-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
; A9--   | &26 | &1E | &16 | &0E | &06 | &01 | &09 | &11 | &19 | &21 |   --A14
;        |  A  |  S  |  D  |  F  |  G  |  H  |  J  |  K  |  L  |ENTER|
;      +-+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-+
; A8-- |  &27  | &1F | &17 | &0F | &07 | &00 | &08 | &10 | &18 |  &20  | --A15
;      | SHIFT | SYM |  Z  |  X  |  C  |  V  |  B  |  N  |  M  | SPACE |
;      +-------+-----+-----+-----+-----+-----+-----+-----+-----+-------+
;          |      |     |     |     |     |     |     |     |      |
;          D0     D1    D2    D3    D4    D4    D3    D2    D1     D0
;
; See also:
;  https://mdfs.net/Docs/Comp/Spectrum/Keyboard/KbdKeyNums
