; BBCPLOT.ASM
; -----------
; Implement BBC PLOT actions on ZX Spectrum - J.G.Harston
;
; PLOTK=action %0pxxxraa
;               ------00 move
;               ------xx plot in foreground
;               -----0-- relative coords
;               -----1-- absolute coords
;               -0------ draw a line
;               -1000--- plot a point
; PLOTX=X coord 0-1023 (32*8*4)
; PLOTY=Y coord 0-767  (24*8*4)
;
; Size: about 256 bytes

VDUPLOT	LD	A,(PLOTK)
	BIT	2,A
	JR	NZ,VPLOT2	; Already absolute
	LD	HL,(PLOTX)	; Convert to absolute
	LD	DE,(LASTX)
	ADD	HL,DE
	LD	(PLOTX),HL
	LD	HL,(PLOTY)
	LD	DE,(LASTY)
	ADD	HL,DE
	LD	(PLOTY),HL
VPLOT2	AND	3
	CALL	NZ,VPLOT3	; Not %xxxxxx00 - not MOVE
	LD	HL,LASTY+1
	LD	DE,PREVY+1
	LD	BC,8
	LDDR			; Copy PREV<-LAST<-PLOT
	RET

VDIFF	JP	P,VDIFF1	; P/M set on entry from HL
	INC	HL		; Adjust for negative diff
	INC	HL
	INC	HL
VDIFF1	BIT	7,D
	JR	Z,VSCALE
	INC	DE		; Adjust for negative diff
	INC	DE
	INC	DE
VSCALE	SRA	H		; HL=HL/4
	RR	L
	SRA	H
	RR	L
	SRA	D		; DE=DE/4
	RR	E
	SRA	D
	RR	E
	RET

VPLOT3	LD	A,(PLOTK)
	CP	&48
	RET	NC
	CP	&40
	JR	C,VDRAWXY	; %00xxxxxx=draw a line
;				; %01000xxx=plot a point
VPLOTXY	LD	DE,(PLOTX)	; DE=X 0-1024
	LD	HL,(PLOTY)	; HL=Y 0-768
	CALL	VSCALE		; Scale to 0-255,0-191
	LD	A,D		; High byte of X
	AND	A
	RET	NZ		; X<0 or X>&00FF
	LD	A,H		; High byte of Y
	AND	A
	RET	NZ		; Y<0 or Y>&00FF
	LD	C,E
	LD	B,L		; BC=(Y,X)
VPLOTBC	LD	A,&BF
	SUB	B
	RET	C		; Y out of range
	CALL	&22B0		; Convert to HL/A
	JP	&22EC		; Jump to plot pixel using ATTRT

VDRAWXY	LD	HL,(LASTX)
	LD	DE,(LASTY)
	CALL	VSCALE
	LD	H,E
	LD	(23677),HL	; Current point in internal coords
	LD	HL,(PLOTX)
	LD	DE,(LASTX)
	AND	A
	SBC	HL,DE
	EX	DE,HL		; DE=offset X
	LD	HL,(PLOTY)
	LD	BC,(LASTY)
	AND	A
	SBC	HL,BC		; HL=offset Y
	CALL	VDIFF		; Scale as difference
; HL=&FF00...&00FF Y = -255 to +255
; DE=&FF00...&00FF X = -255 to +255
	LD	A,E		; A=X
	LD	E,D		; E=&FF or &00
	SLA	E		; E=&FE or &00
	JR	Z,VDRW2
	NEG
VDRW2	INC	E		; E=&FF or &01
	LD	C,A		; C=ABS(X)
	LD	A,L		; A=Y
	LD	D,H		; D=&FF or &00
	SLA	D		; D=&FE or &00
	JR	Z,VDRW3
	NEG
VDRW3	INC	D		; D=&FF or &01
	LD	B,A		; B=ABS(Y)

; Enter here: BC=value of relative coords 0 to 255
;             DE=sign of relative coords, &01/&FF
GRDRAW	LD	A,C		; Now as per ROM code
	CP	B
	JR	NC,GDLXGEY
	LD	L,C
	PUSH	DE
	XOR	A
	LD	E,A
	JR	GDLARGE
GDLXGEY	OR	C
	RET	Z
	LD	L,B
	LD	B,C
	PUSH	DE
	LD	D,0
GDLARGE	LD	H,B
	LD	A,B
	RRA
GDLOOP	ADD	A,L
	JR	C,GDLDIAG
	CP	H
	JR	C,GDLHRVT
GDLDIAG	SUB	H
	LD	C,A
	EXX
	POP	BC
	PUSH	BC
	JR	GDLSTEP
GDLHRVT	LD	C,A
	PUSH	DE
	EXX
	POP	BC
GDLSTEP	PUSH	HL
	INC	C		; Adjust to set flags
	LD	HL,(23677)	; Get current coords
	LD	A,B		; Add relative coords
	ADD	A,H
	LD	B,A
	LD	A,C
	ADD	A,L
	LD	C,A		; Return absolute coords
	JR	C,GDLRANG
	JR	Z,GDLEXIT
GDLPLOT	DEC	A		; Reverse the adjust
	LD	C,A
	LD	(23677),BC	; Update COORDS
	CALL	VPLOTBC		; Plot the point
	POP	HL
	EXX
	LD	A,C
	DJNZ	GDLOOP
	POP	DE
	RET
GDLRANG	JR	Z,GDLPLOT
GDLEXIT	POP	HL
	EXX
	POP	DE
	RET

PLOTK	DEFB	0
PLOTX	DEFW	0
PLOTY	DEFW	0
LASTX	DEFW	0
LASTY	DEFW	0
PREVX	DEFW	0
PREVY	DEFW	0
