; PLOT.ASM
; ZX Spectrum graphics routines using whole screen - J.G.Harston
; --------------------------------------------------------------
; GRMOVA, GRMOVR - Move to a point, absolute/relative coords
; GRPLTA, GRPLTR - Plot a point, absolute/relative coords
; GRDRWA, GRDRWR - Draw a line, absolute/relative coords
;
; Absolute coordinates are &00 to &FF representing 0 to 255
; Relative coordinates are &81 to &7F representing -127 to +127
;
; On entry: BC=plot coordinate, C=X, B=Y
; On exit:  AF,BC,DE,HL corrupted
;           AF',BC',DE' also corrupted by DRAW
; Size:     125 bytes


; Move to BC=(Y,X)
; ----------------
GRMOVR	CALL	GRADJST		; Convert relative coords
GRMOVE
GRMOVA	LD	(23677),BC	; MOVE to (C,B)
	RET

; Plot point at BC=(Y,X)
; ----------------------
GRPLTR	CALL	GRADJST		; Convert relative coords
GRPLOT
GRPLTA	LD	(23677),BC	; MOVE to (C,B)
	LD	A,&BF
	SUB	B
	RET	C		; Y out of range
	CALL	&22B0		; Convert to HL/A
	JP	&22EC		; Jump to plot pixel

GRADJST	LD	HL,(23677)	; Get current coords
	LD	A,B		; Add relative coords
	ADD	A,H
	LD	B,A
	LD	A,C
	ADD	A,L
	LD	C,A		; Return absolute coords
	RET

; Draw line from current point to BC=(Y,X)
; ----------------------------------------
; Enter here: BC=absolute coords
;
GRDRWA	LD	HL,(23677)	; Current COORDS
	LD	DE,&0101	; +ve increments
	LD	A,B		; Convert to relative Y
	SUB	H
	JR	NC,GRDRW1	; Positive movement
	NEG			; Negative movement
	LD	D,&FF
GRDRW1	LD	B,A
	LD	A,C		; Convert to relative X
	SUB	L
	JR	NC,GRDRW2	; Positive movement
	NEG			; Negative movement
	LD	E,&FF
GRDRW2	LD	C,A
	JR	GDRAW2

; Enter here: BC=7-bit relative coords -128 to +127
;
GRDRWR	LD	DE,&0101	; +ve increments
	LD	A,B
	AND	A
	JP	P,GDRAW1	; Y is positive
	NEG			; Negate Y
	LD	B,A
	LD	D,&FF		; -ve increment
GDRAW1	LD	A,C
	AND	A
	JP	P,GDRAW2	; X is positive
	NEG			; Negate X
	LD	C,A
	LD	E,&FF		; -ve increment

; Enter here: BC=value of relative coords 0 to 255
;             DE=sign of relative coords, &01/&FF
;
GRDRAW
GDRAW2	LD	A,C		; Now as per ROM code
	CP	B
	JR	NC,GDLXGEY
	LD	L,C
	PUSH	DE
	XOR	A
	LD	E,A
	JR	GDLARGE
GDLXGEY	OR	C
	RET	Z
	LD	L,B
	LD	B,C
	PUSH	DE
	LD	D,0
GDLARGE	LD	H,B
	LD	A,B
	RRA
GDLOOP	ADD	A,L
	JR	C,GDLDIAG
	CP	H
	JR	C,GDLHRVT
GDLDIAG	SUB	H
	LD	C,A
	EXX
	POP	BC
	PUSH	BC
	JR	GDLSTEP
GDLHRVT	LD	C,A
	PUSH	DE
	EXX
	POP	BC
GDLSTEP	PUSH	HL
	INC	C		; Adjust to set flags
	CALL	GADJUST		; Add increment to COORDS
	JR	C,GDLRANG
	JR	Z,GDLEXIT
GDLPLOT	DEC	A		; Reverse the adjust
	LD	C,A
	CALL	GRPLOT		; Plot the point, updating COORDS
	POP	HL
	EXX
	LD	A,C
	DJNZ	GDLOOP
	POP	DE
	RET
GDLRANG	JR	Z,GDLPLOT
GDLEXIT	POP	HL
	EXX
	POP	DE
	RET

