; RAWIO.ASM
; ZX Spectrum Text I/O routines - J.G.Harston
; -------------------------------------------
; The ZX Spectrum ROM does not have any general-purpose entry
; points for system calls. These routines implement some typical
; text I/O calls that are generally found on other systems.
;
; IOINIT  - Initialise I/O by selecting screen for output.
; IOWRCH  - Write a character to screen.
; IOTEST  - Test for pending keypress.
; IORDCH  - Wait for keyboard character.
;
; Requires  IY=>system variables, interupts on, IRQ routine at #38 running.
; Size:   25 bytes.


; Exported values:
IOCHDEL  EQU	0		; No delete output character
IOKYDEL  EQU	12		; CHR$12 is DELETE input key
IOKYBS   EQU	0		; No BACKSPC key
IOCRLF   EQU	1		; CR generates automatic LF


; IOINIT - Initialise screen driver
; ---------------------------------
; On exit,  AF,BC,DE,HL corrupted
IOINIT	LD	A,2
	JP	&1601

; IOWRCH - Write raw character
; ----------------------------
; On entry, A =character
; On exit,  AF=corrupted
;           Alternate registers corrupted
IOWRCH	EQU	&0010

; IOTEST - Test for input character
; ---------------------------------
; On exit, Z =no keypress pending
;          NZ=keypress pending
IOTEST	BIT	5,(IY+1)
	RET

; IORDCH - Wait for raw input character
; -------------------------------------
; On exit,  A=character
IORDCH	HALT
	CALL	IOTEST
	JR	Z,IORDCH
	RES	5,(IY+1)
	LD	A,(23560)
	AND	A
	RET
