; READLN.ASM
; ZX Spectrum Text input routine - J.G.Harston
; --------------------------------------------
; On entry  HL=address to enter text.
;            B=if enabled, maximum line length, excluding <cr>.
; On exit   HL=address of text, terminated with <cr>.
;            B=length of text, excluding <cr>, up to 255 characters.
;           Cy: clear=ok, set=BREAK.
;           Alternate registers corrupted.
; Requires  IY=>system variables.
;           IORDCH: routine to return keyboard character.


; Options
IOMAXLN  EQU	1		; Enable maximum length


IOINPUT	PUSH	HL		; Save start of text
  IF IOMAXLN
	LD	C,B		; C=maximum length
  ENDIF
	LD	B,0		; Initialise zero length
IORDGET	CALL	IORDCH		; Wait for character
	JR	C,IORDBRK	; BREAK pressed
	CP	12
	JR	Z,IORDDEL	; DELETE pressed
	CP	13
	JR	Z,IORDENT	; ENTER pressed, done
	CP	32
	JR	C,IORDGET	; Wait for character
	LD	(HL),A		; Store character
  IF IOMAXLN
	LD	A,B
	CP	C
	JR	NC,IORDGET	; Too long
	INC	B		; Increment length
	LD	A,(HL)		; Get character back
  ELSE
	INC	B		; Increment length
	JR	Z,IORDBAK	; Too long, skip back
  ENDIF
	RST	16		; Print character
	INC	HL		; Point to next location
	JP	IORDGET		; Loop for next
IORDDEL	LD	A,B
	AND	A
	JR	Z,IORDGET	; Nothing to delete
	LD	A,8		; Backspace
	RST	16
	LD	A,32		; Overwrite character
	RST	16
	LD	A,8		; Backspace
	RST	16
	DEC	HL		; Decrement pointer
IORDBAK	DEC	B		; Decrement length
	JR	IORDGET		; Loop for next
IORDENT	LD	(HL),A		; Insert terminating <cr>
	RST	16		; Print <cr>, returns Carry=NC
IORDBRK	POP	HL		; Get text pointer back
	RET			; Exit
