; TEXTIO.ASM
; ZX Spectrum Text I/O routines - J.G.Harston
; -------------------------------------------
; The ZX Spectrum ROM does not have any general-purpose entry
; points for system calls. These routines implement some typical
; text I/O calls that are generally found on other systems. This
; library uses native Spectrum character codes.
;
; IOINIT  - Initialise I/O by selecting screen for output.
; IOWRCH  - Write a character to screen.
; IOTEST  - Test for pending keypress.
; IORDCH  - Wait for keyboard character, allowing symbol characters.
;
; Requires  IY=>system variables, interupts on, IRQ routine at #38 running.


; Options:
;IOCURSR EQU	143		; Cursor character
;IOCLICK EQU	0		; Key click
IOCURSR  EQU	1		; Cursor character, 1=show CAPS, 0=off
IOCLICK  EQU	1		; Key click, 0=no click
IOBREAK  EQU	1		; BREAK tested for
IOMAXLN  EQU	1		; Enable maximum length

; Exported values:
IOCHDEL  EQU	0		; No delete output character
IOKYDEL  EQU	12		; CHR$12 is DELETE input key
IOKYBS   EQU	0		; No BACKSPC key
IOCRLF   EQU	1		; CR generates automatic LF


; IOINIT - Initialise screen driver
; ---------------------------------
; On exit,  AF,BC,DE,HL corrupted
IOINIT	LD	A,2
	JP	&1601

; IOWRCH - Write character
; -----------------------------
; On entry, A =character
; On exit,  AF=corrupted
;           Alternative registers corrupted
IOWRCH	EQU	16

; IOTEST - Test for input character
; ---------------------------------
; On exit,  Z =no keypress pending
;           NZ=keypress pending
IOTEST	BIT	5,(IY+1)
	RET

; IORDCH - Wait for input character
; ---------------------------------
; On exit,  A =character
;           NC=character returned
;           CS=BREAK pressed, A=undefined
;           Alternative registers corrupted
IOCAPS	LD	A,(23658)
	XOR	8		; Toggle CAPSLOCK
	LD	(23658),A
IORDCH
  IF IOCURSR & 224
	LD	A,IOCURSR
	RST	16		; Output cursor
  ENDIF
  IF IOCURSR - 1
  ELSE
  	SET	7,(IY+85)	; Flashing
  	LD	A,&43
  	BIT	3,(IY+48)	; Test CAPS
  	JR	NZ,IORDNXT
  	LD	A,&4C
IORDNXT	RST	16		; Output cursor
  	RES	7,(IY+85)
  ENDIF
  IF IOCURSR
	LD	A,8
	RST	16		; Back over cursor
  ENDIF
IORDLP	HALT			; Wait for interupt
  IF IOBREAK
	CALL	&1F54		; Test BREAK key
	CCF
	JR	C,IORDOK	; BREAK pressed
  ENDIF
	BIT	5,(IY+1)
	JR	Z,IORDLP	; Loop until key pressed
	LD	A,(23560)	; Get keypress
	RES	5,(IY+1)	; Set 'no key pressed'
	CP	6
	JR	Z,IOCAPS	; CAPS pressed
  IF IOCLICK
  	PUSH	AF
  	PUSH	BC
	PUSH	HL
	PUSH	IX
	LD	D,0
	LD	E,(IY-1)
	LD	HL,&00C8
	CALL	&03B5		; Keyclick
	POP	IX
	POP	HL
	POP	BC
	POP	AF
  ENDIF
	AND	A		; NC, P=char<128
	JP	P,IORDOK	; Character pressed
	SUB	194
	JR	NC,IOSYMB
	LD	A,2
IOSYMB	AND	15		; Convert symbol key
	ADD	A,IOCONV & 255	; Index into conversion table
	PUSH	HL
	LD	L,A
	LD	H,IOCONV / 256
	LD	A,(HL)
	POP	HL
IORDOK
  IF IOCURSR
	PUSH	AF		; Save character and BREAK/OK flag
	LD	A,32		; Overwrite cursor
	RST	16
	LD	A,8		; Backspace
	RST	16
	POP	AF
  ENDIF
	RET
; *NB* Table must not wrap over page boundary
IOCONV	DEFB	126,124,127,93,91,0	; ~ | (C) ] [ .
	DEFB	0,0,0,125,123,92	; . .  .  } { \
